function [EMg]=atteso(X,Y,GammaD,PostGamD,Hg,ndist)

%atteso:  Bayesian Variable Selection
%         Approximation of the expected value of the regression coeffs 
%	  Bayes estimate 
%***********************************************************************
%INPUTS:
% X,Y,		training data
% GammaD,	distinct models visited by the MCMC search ordered
%		according to their posterior probability (cell or matrix)
% PostGamD, 	posterior probability of visited models
% Hg,		hyperparameter of the prior on regression coefficients
%		- scalar (g-prior)
%		- vector of std's (diagonal prior)
%		- full matrix (non-diagonal prior)
% ndist,	number of models in GammaD to be used in the approx.
%
%OUTPUTS:
% EMg,		approx expected value of the regression coeffs 
%		Bayes estimate
%
%USAGE:
% [EMg]=atteso(X,Y,GammaD,PostGamD,Hg,ndist)
%
%NOTES:
% X,Y training dataset
% The approximation is computed by averaging over the best 'ndist' 
%  visited models by the MCMC search.
%
%
%Copyright (c) 1997-2003 Marina Vannucci.
%**********************************************************************

[p]=size(X,2);       
[q]=size(Y,2);           
EMg=zeros(p,q);  	         % Dimensions of EMg are p times q 		 	
costante=sum(PostGamD(1:ndist));
	
for i=1:ndist
 	switch class(GammaD)
		case 'cell'
			I=str2num(GammaD{i});
		otherwise
			I=find(GammaD(i,:));
	end	
	Xg=X(:,I);
	Mg=zeros(p,q);
	if (size(Hg,1)==1 & size(Hg,2)==1)
			%--------------------------- Hg scalar - gPrior 
		Hg2=Hg/(Hg+1);
		Mg(I,:)=Hg2*inv(Xg'*Xg)*Xg'*Y;
							
	elseif (size(Hg,1)==1 | size(Hg,2)==1)	
			%--------------------------- Hg vector of std's 
			
		Hg2=diag(Hg(I).^2);
		Mg(I,:)=inv(eye(length(I))+Hg2*Xg'*Xg)*Hg2*Xg'*Y;
	else
			%--------------------------- Hg full matrix
		Hg2=Hg(I,I);
		Mg(I,:)=inv(eye(length(I))+Hg2*Xg'*Xg)*Hg2*Xg'*Y;			  
	end
	EMg=EMg + Mg*PostGamD(i)/costante;

end
