function [Uni,UniD,GammaD,logProb,logProbD,PostGamD,AD,S]= ...
bvsme_fast(gamprec,X,Y,delta,k,w,Hg)

%bvsme_fast: Bayesian Variable Selection with Metropolis search,
% 	     g-prior or selection prior with diagonal and non-diagonal
%	     cov. matrix and fast form of QR matrices updating
%			     Main Program 
%***********************************************************************
%INPUTS:
% gamprec,      starting binary vector.
%               If gamprec=[] the program asks for r to select a starting
%               vector with r randomly chosen elements equal to 1 
% X,            independent variables, matrix (n by p)
% Y,            response variables, matrix (n by q)
% delta, k,     hyperparameters of Inverse Wishart prior
% w,		hyperparameter of prior on latent binary vector
%		- vector of parameters w_j, j=1...p (Bernoulli priors)
%		- scalar in the case of equal parameters w_j=w 
%		  (Binomial prior with mean pw)
%		- vector of two values for a Beta-Binomial prior
% 		(with mean=pw(1), w(2)=sum of the two parameters of the Beta)
% Hg,   	hyperparameter of the prior on regression coefficients
%		- vector of std's 
%		- scalar for g-prior
%		- full var-cov matrix
%
%OUTPUTS:
% Uni,		vector with number of ones of visited models
% UniD,		vector with number of ones of distinct visited models
% GammaD,	distinct visited models, ordered according to
%		their (normalized) relative posterior probability
%		(cell array of strings)
% logProb,      log-relative posterior probabilities 
%		of all visited models
% logProbD      log-relative posterior probabilities
%               of distinct visited models
% PostGamD	normalized ordered relative probabilities 
%		of distinct visited models 
% AD, S,	iteration numbers of accepted additions/deletions (AD) 
%		and swaps (S)
%USAGE:
% [Uni,UniD,GammD,logProb,logProbD,PostGamD,AD,S]= ...
%	bvsme_fast(gamprec,X,Y,delta,k,w,Hg)
%
%NOTES:
% Data must be centered.
% The programs asks for the Metropolis parameters
%  (initial number of variables included, probabilities of  
%  deletion/addition and swap moves, total number of iterations)
% QRdelete-QRinsert are used to update QR matrices (not for Hg full matrix)
% The code implements a fast form of updating QR matrices
%  (not for gPrior)
% QR matrices recomputed every 500 iterations
% GammaD is in cell array format 
%  (elements are strings --> required by unique.m)
% Search for replicates among visited accepted models of same size
% Functions called: gofg_fast, metro_fast, replichef, probordf
%
%REFERENCE: see readme_fast
%
%
%Copyright (c) 1997-2003 Marina Vannucci.
%**************************************************************************

[nOss p]=size(X);

[q]=size(Y,2); 
        % p independent variables, q responses, nOss observations

disp(' ')
disp('------- METROPOLIS --------')
disp(' ')


if isempty(gamprec)
	disp(' ')
	disp('-- Starting binary vector with r randomly chosen elements equal to 1');
	r=input('   please provide r, r=?');	
	randp=randperm(p);
	gamprec=zeros(1,p);
	onenum=randp(1:r);
	gamprec(onenum)=1;
else
end

disp(' ')
disp('-- The Metropolis search moves by')
disp('   A/D)adding or deleting a variable (with probability P)')
disp('                            or') 
disp('   S)swapping two variables (with probability 1-P)')
pPar=input('   please provide P, probabitity of moves A/D, P=?');
disp(' ')
nmetr=input('-- Number ot total iterations (multiple than 500)=?');	
disp(' ')
disp('------- RUNNING METROPOLIS ...')
disp(' ')

nPar=500;

if (size(Hg,1)==1 & size(Hg,2)==1)
%--------------------------- Hg scalar - gPrior - no augmenting/fast updating

	inidim=[];
	Ytilde=Y;

elseif (size(Hg,1)==1 | size(Hg,2)==1)
%--------------------------- Hg vector of std's - augmenting and fast updating

	inidim=max(20,length(find(gamprec)));
	Ytilde=cat(1,Y, zeros(inidim,q));			 	
else
%--------------------------- Hg full matrix - augmenting without QRinsert/delete
	inidim=[];
	Ytilde=Y;				 
end		

%Gamma{1}=num2str(find(gamprec)); 
	%UNCOMMENT THIS IF YOU WANT TO SAVE ALL VISITED MODELS
GammaD{1}=num2str(find(gamprec));
Uni(1)=r;
UniD(1)=r;
logProb=zeros(1,nmetr+1);
AD=[];
S=[];
conta=1;
contaD=1;
while conta<nmetr+1

%-------------------- updating QR decomposition
	
	[probprec, Qqr, Rqr,IRES1,IRES2] = ...
		gofg_fast(gamprec,inidim,1, X,Ytilde,[],[],[],[],...
		[],k,delta,w,Hg);		
	logProb(conta)= probprec;
	logProbD(contaD)=probprec;

%-------------------- Metropolis (nPar iterations)

       	[UniSt,ProbSt, GammaSt, storeAD, storeS]= metro_fast(inidim, ....
	gamprec,probprec,X,Ytilde,Qqr,Rqr,IRES1,IRES2,k,delta,w,Hg,nPar,pPar);
%	Gamma=cat(1,Gamma,GammaSt);
	%UNCOMMENT THIS IF YOU WANT TO SAVE ALL VISITED MODELS
	logProb(conta+1:conta+nPar)=ProbSt;
	Uni=[Uni UniSt];
	indiciD=sort([storeAD storeS]);
	GammaD=cat(1,GammaD, GammaSt(indiciD));
	logProbD=[logProbD, ProbSt(indiciD)];	
	UniD=[UniD, UniSt(indiciD)];
	AD=cat(2, AD, (conta-1)+storeAD);
	S=cat(2, S, (conta-1)+storeS);
	conta=conta+nPar;
	contaD=length(logProbD);
	gamprec=zeros(1,p);
	gamprec(str2num(GammaSt{nPar}))=ones(1,length(str2num(GammaSt{nPar})));
	disp(' ')
       	disp('Number of iterations:')
       	disp(conta-1)
       	disp('...')
	if (size(Hg,1)==1 & size(Hg,2)==1)
	elseif (size(Hg,1)==1 | size(Hg,2)==1) %--- Hg vector of std's 
%-------------------- Update of dimensions for IRES1 and IRES2	
		media=ceil(mean(UniSt));
		inidim=max(media,length(find(gamprec)));
		Ytilde=cat(1,Y, zeros(inidim,q));
		else
	end
end
disp(' ')
disp('------- Searching for replicates ...')
disp(' ')

%------------------- Search for replications
[UniD,GammaD, logProbD]=replichef(UniD,GammaD, logProbD);

%------------------- Normalized posterior and  marginal probs.
%                    Ordering of distinct visited vectors according
%                    to probability

[UniD,GammaD, logProbD, PostGamD]=probordf(UniD,logProbD, GammaD);


disp(' ')
disp('--- End.') 
