function [r, Qqr, Rqr, IRES1,IRES2] = gofg_fast(...
gamma, inidim,indice, X,Y, QqrP,RqrP,IRES1P,IRES2P,elem, k,delta, w,Hg)

%gofg_fast: log-relative posterior probability of gamma. 
%	    Selection prior is with fast form of updating
%*******************************************************************
%INPUTS:
% gamma,          binary vector
% inidim	  maximum dimension for the fast form of updating	
% indice,         1 -- QR decomp
%                 2 -- QRinsert
%                 3 -- QRdelete
% X, 		  independent variables, n by p
% Y,              augmented response matrix, (n+p) by q
% QqrP,RqrP,      QR-matrices of the previous vector
% IRES1P,IRSE2P,  binary matrices for fast form of updating
% elem,           position to add/delete
% k,delta,        hyperparameters of Inverse Wishart prior
% w,              hyperparameters of prior on latent binary vector
% Hg,             hyperparameter of prior on regression coefficients 
%		  (vector or scalar or matrix )
%
%OUTPUTS:
% r,              log-relative posterior probability of gamma
% Qqr, Rqr,       QR matrices of gamma
% IRES1,IRES2,	  binary matrices for fast form of updating
%
%USAGE:
% [r, Qqr, Rqr, IRES1,IRES2]= gofgme_fast(...
% gamma,inidim,indice,X,Y,QqrP,RqrP,IRES1P,IRES2P,elem,delta,k,w,Dg);
%
%NOTES:
% Called by bvsme_fast and metro_fast
%
%REFERENCE: see readme_fast
%
%Copyright (c) 1997-2003 Marina Vannucci.
%**********************************************************************

[nOss p]=size(X);
[q]=size(Y,2);
Qk=k*eye(q);                         
I=find(gamma);
nonnulle=length(I);
switch indice
	case 1	% ------------------ Indice = 1 -- QR Decomposition

		XGam=X(:,I);
		if (size(Hg,1)==1 & size(Hg,2)==1) % g-prior
			IRES1=[];
			IRES2=[];
			[Qqr Rqr]=qr(XGam);

		elseif (size(Hg,1)==1 | size(Hg,2)==1) % diagonal prior	
				
			IRES1=eye(inidim);
			IRES2=IRES1(:,1:nonnulle);	 
			IRES1(:,1:nonnulle)=[]; 
			moltiplica=diag(Hg(I));		
			Xtilde=cat(1,XGam*moltiplica, IRES2);
			[Qqr Rqr]=qr(Xtilde);  

		else  % non-diagonal prior	
			inidim=nonnulle;
			IRES1=[];
			IRES2=[];	 
			Y=cat(1,Y,zeros(nonnulle,q));
			sotto=eye(nonnulle);
			moltiplica=sqrtm(Hg(I,I));;		
			Xtilde=cat(1,XGam*moltiplica, sotto);
			[Qqr Rqr]=qr(Xtilde); 			
		end             
		
	case 2	%------------------- Indice = 2 -- QRinsert
	
		prima=find(I<elem);
		jj=length(prima)+1;
		if (size(Hg,1)==1 & size(Hg,2)==1) % g-prior
			ins=X(:,elem);
			IRES1=[];
			IRES2=[];
		else			
			sotto=IRES1P(:,1);
			[rR,cR]=size(IRES2P);
			if jj==1 IRES2=[sotto IRES2P];
			elseif jj>cR IRES2=[IRES2P sotto];
			else 
				IRES2=[IRES2P(:,1:jj-1) sotto IRES2P(:,jj:cR)];
			end	
			IRES1=IRES1P;
			IRES1(:,1)=[];
			ins=cat(1, X(:,elem)*Hg(elem), sotto);
		end
		[Qqr Rqr]=qrinsert(QqrP,RqrP,jj,ins);

	case 3	%------------------- Indice = 3 -- QRdelete
		prima=find(I<elem);
		jj=length(prima)+1;
		[Qqr Rqr]=qrdelete(QqrP,RqrP,jj);
		if (size(Hg,1)==1 & size(Hg,2)==1) % g-prior
			IRES1=[];
			IRES2=[];
		else 
			IRES1=[IRES2P(:,jj) IRES1P];
			IRES2=IRES2P;
			IRES2(:,jj)=[];
		end     
end
%---------------------------------- prior of gamma
if (length(w)==1 | length(w)==p)
	prog=sum(log((w.^gamma).*(1-w).^(1-gamma)));
elseif length(w)==2
	alpha=w(2)*w(1);	% Beta prior (mean=pw(1), w(2)=alpha+beta) 
	betapar=w(2)*(1-w(1));
	gami=sum(gamma);
	prog=(gammaln(alpha+gami)+gammaln(p+betapar-gami))- ...
	      gammaln(p+betapar+alpha);
end

%----------------------------------- log-relative posterior prob of gamma

sub=Qqr(:,nonnulle+1:nOss+inidim)'*Y;	% Seber, page 498
RSP=sub'*sub;                       	% residual sum of products

if (size(Hg,1)==1 & size(Hg,2)==1)      % g-prior
	QGam=(Hg/(Hg+1))*RSP + Y'*Y/(Hg+1);
	Qstar=Qk+QGam;
	r= -(q/2)*nonnulle*log(Hg+1)-((delta+nOss+q-1)/2)*log(det(Qstar))+prog;
else	 
	Qstar=Qk+RSP;                   % q by q
	switch nonnulle
		case 1
        		detH0K=Rqr(1,1)^2;
		otherwise
        		detH0K=prod(diag(Rqr).^2);
	end
r= -(q/2)*log(detH0K)-((delta+nOss+q-1)/2)*log(det(Qstar))+prog;
end
