function [UniSt,ProbSt, GammaSt, storeAD, storeS]= ... 
metro_fast(inidim,gamprec,probprec,X,Ytilde,QqrP,RqrP,IRES1P, IRES2P, ...
k,delta,w,Hg,nPar,pPar);

%metro_fast: Metropolis - Selection prior is with fast form of updating
%**********************************************************************
%INPUTS:
% inidim	  maximum dimension for the fast form of updating	
% gamprec,        initial binary vector.
% probprec,	  log-relative posterior probability of gamprec
% X,              independent variables, n by p
% Ytilde,         augmented response matrix, (n+inidim) by q
% QqrP,RqrP,	  QR-matrices for gamprec
% IRES1P,IRSE2P,  binary matrices for fast form of updating
% k,delta,        hyperparameters Inverse Wishart
% w,              hyperparameter (Bernoulli priors)
% Hg,		  hyperparameter - prior on regression coefficients
% nPar,           number of iterations
% pPar,           probabitity of Addition/Deletion
%
%OUTPUTS:
% ProbSt,	  log-relative posterior probabilities
%                 of visited vectors
% GammaSt,        cell with positions of 1's in visited vectors
% storeAD,storeS, iteration numbers of accepted additions/deletions (AD)
%                 and swaps (S)
%
%USAGE:
% [ProbSt, GammaSt, storeAD, storeS]= metro_fast(...
% gamprec,probprec,X,Ytilde,QqrP,RqrP,IRES1P,IRES2P,k,delta,w,Hg,nPar,pPar);
%
%NOTES:
% Called by bvsme_fast
% Functions called: gofg_fast
% An adding move is forced with empty model
% A delete move is forced with full model
% 3 possible priors are possible: g-prior (Hg scalar), diagonal prior
% (Hg vector of std's) and non-diagonal prior (Hg full var-cov matrix).
% With full prior QRdelete/insert updating is not possible.
%
%REFERENCE: see readme_fast
%  
%     
%Copyright (c) 1997-2003 Marina Vannucci.
%**********************************************************************

[nOss p]=size(X);
[q]=size(Ytilde,2);
storeAD=[];
storeS=[];
GammaSt=cell(nPar,1);
ProbSt=zeros(1,nPar);
UniSt=zeros(1,nPar);

for i=1:nPar
	flag=0;
	gamnew=gamprec;
	poszeros=find(gamprec==0);
	if length(poszeros)==p			% Forced to add variable
		chi='AD';
		choice=ceil(rand(1)*p);
		gamnew(choice)=1;
		if (size(Hg,1)==1 & size(Hg,2)==1) %------ gPrior
			
			[probnew, QqrN, RqrN,IRES1N,IRES2N] = ...
			gofg_fast(gamnew,inidim,2, X, Ytilde, ...
			QqrP, RqrP,[],[], choice, k,delta, w,Hg);	
	
		elseif (size(Hg,1)==1 | size(Hg,2)==1)%---diagonal prior
			
%-------------------Reset of limit size of IRES1 and IRES2		
			if size(IRES1P,2)==0
				flag=1;
				Ytilde(nOss+inidim+1,:)=zeros(1,q);
				inidim=inidim+1;
				[probnew, QqrN, RqrN,IRES1N,IRES2N] = ...
				gofg_fast(gamnew,inidim,1, X,Ytilde, ...
				[],[],[],[],[],k,delta,w,Hg);
			else				
				[probnew, QqrN, RqrN,IRES1N,IRES2N] = ...
				gofg_fast(gamnew,inidim,2, X, Ytilde, ...
				QqrP, RqrP,IRES1P,IRES2P, choice, ...
				k,delta, w,Hg);		
			end
		else    %-------- non-diagonal prior
						
			[probnew, QqrN, RqrN,IRES1N,IRES2N] = ...
			gofg_fast(gamnew,inidim,1, X, Ytilde, ...
			[],[],[],[],[],k,delta, w,Hg);		
		end	
	elseif 	isempty(poszeros)              % Forced to delete variable
		chi='AD';
		choice=ceil(rand(1)*p);
		gamnew(choice)=0;
		if ~ismember(1,size(Hg))  %--- non-diagonal prior

			[probnew, QqrN, RqrN,IRES1N,IRES2N] = ...
                       	gofg_fast(gamnew,inidim,1, X, Ytilde, [], [], ...
			[], [],[], k,delta,w,Hg);

		else    %----- diagonal prior or g-prior
						
			[probnew, QqrN, RqrN,IRES1N,IRES2N] = ...
                       	gofg_fast(gamnew,inidim,3, X, Ytilde, QqrP, RqrP, ...
			IRES1P, IRES2P,choice, k,delta,w,Hg);
		end	
		
	else			
		x=rand(1);
		if x<pPar

%-------------------Adding/Deleting
			chi='AD';
			choice=ceil(rand(1)*p);
			if gamnew(choice)==0
				gamnew(choice)=1;	
				if (size(Hg,1)==1 & size(Hg,2)==1) %-- gPrior
			
					[probnew,QqrN,RqrN,IRES1N,IRES2N] = ...
					gofg_fast(gamnew,inidim,2,X,Ytilde, ...
					QqrP,RqrP,[],[],choice,k, ...
					delta,w,Hg);	
	
				elseif (size(Hg,1)==1 | size(Hg,2)==1)
							%--- diagonal prior
			
%-------------------Reset of limit size of IRES1 and IRES2		
					if size(IRES1P,2)==0
						flag=1;
						Ytilde(nOss+inidim+1,:)=zeros(1,q);
						inidim=inidim+1;
						[probnew,QqrN,RqrN,IRES1N,IRES2N] = ...
						gofg_fast(gamnew,inidim,1,X, ...
						Ytilde,[],[],[],[],[],k,delta,w,Hg);
					else				
						[probnew,QqrN,RqrN,IRES1N,IRES2N] = ...
						gofg_fast(gamnew,inidim,2,X, ...
						Ytilde,QqrP,RqrP,IRES1P,IRES2P,... 
						choice,k,delta,w,Hg);		
					end
				else    %-------- non-diagonal prior
					
					[probnew, QqrN, RqrN,IRES1N,IRES2N] = ...
					gofg_fast(gamnew,inidim,1,X, ...
					Ytilde,[],[],[],[],[],k, delta,w,Hg);					
				end	
			else
				gamnew(choice)=0;
				if ~ismember(1,size(Hg)) %-- non-diagonal prior

					[probnew,QqrN,RqrN,IRES1N,IRES2N] = ...
                       			gofg_fast(gamnew,inidim,1,X,Ytilde, ...
					[],[],[],[],[],k,delta,w,Hg);

				else    %----- diagonal prior or g-prior
						
					[probnew,QqrN,RqrN,IRES1N,IRES2N] = ...
                       			gofg_fast(gamnew,inidim,3,X,Ytilde, ...
					QqrP,RqrP,IRES1P,IRES2P,choice,k, ...
					delta,w,Hg);
				end	
			end
		else

%-------------------Swapping
			chi='S';
			poszeros=find(gamprec==0);
			posones=find(gamprec==1);
			choice1=ceil(rand(1)*length(poszeros));
			choice2=ceil(rand(1)*length(posones));
			elem=posones(choice2);
			gamnew(elem)=0;
			if ~ismember(1,size(Hg))  %--- non-diagonal prior

				[probnew, QqrN, RqrN,IRES1N,IRES2N] = ...
                     	  	gofg_fast(gamnew,inidim,1,X,Ytilde,[],[], ...
				[], [],[], k,delta,w,Hg);

			else    %----- diagonal prior or g-prior
						
				[probnew, QqrN,RqrN,IRES1N,IRES2N]=gofg_fast(...
				gamnew,inidim,3,X,Ytilde,QqrP,RqrP,IRES1P, ...
				IRES2P,elem,k,delta,w,Hg);
			end	
               	 	elem=poszeros(choice1);  
			gamnew(elem)=1;
			if ~ismember(1,size(Hg))  %--- non-diagonal prior

				[probnew, QqrN, RqrN,IRES1N,IRES2N] = ...
                       		gofg_fast(gamnew,inidim,1,X,Ytilde,[],[], ...
				[], [],[], k,delta,w,Hg);

			else    %----- diagonal prior or g-prior
			
				[probnew,QqrN,RqrN,IRES1N,IRES2N]=gofg_fast(...
				gamnew,inidim,2,X,Ytilde,QqrN,RqrN,IRES1N, ...
				IRES2N,elem,k,delta,w,Hg);
			end
		end
	end	
	move=min(0, (probnew-probprec) );
	if move==0
		eval(['store',chi,'=[store',chi,' i];'])
		QqrP=QqrN;
                RqrP=RqrN;
		IRES1P=IRES1N;
		IRES2P=IRES2N;
                probprec=probnew;
                gamprec=gamnew;   
                ProbSt(i)=probnew;     
                GammaSt{i}=num2str(find(gamnew));
		UniSt(i)=length(find(gamnew));
	else if (rand(1) < exp(move))
			eval(['store',chi,'=[store',chi,' i];'])
			QqrP=QqrN;
			RqrP=RqrN;
			IRES1P=IRES1N;
			IRES2P=IRES2N;
			probprec=probnew;
			gamprec=gamnew;
			ProbSt(i)=probnew;
			GammaSt{i}=num2str(find(gamnew));		
			UniSt(i)=length(find(gamnew));
		else
			ProbSt(i)=probprec; 
			GammaSt{i}=num2str(find(gamprec));
			UniSt(i)=length(find(gamprec));
		
			if (size(Hg,1)==1 | size(Hg,2)==1) %---- diag prior
				if flag==1 
					Ytilde(nOss+inidim,:)=[];
					inidim=inidim-1; 
				else
				end
			else	
			end
		end
	end
end

